/*
 * MIDPath - Copyright (C) 2006-2008 Guillaume Legris, Mathieu Legris
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 only, as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License version 2 for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * version 2 along with this work; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA 
 */
package org.thenesis.microbackend.ui.graphics;

import org.thenesis.microbackend.ui.Logging;


public class VirtualFontImpl implements VirtualFont {

	private static final int FONT_8X8 = 0;
	private static final int FONT_8X16 = 1;
	private static final int FONT_DEFAULT = FONT_8X16;
	private int font = FONT_DEFAULT;

	private int inset = 1;
	private int size;
	
	VirtualFontImpl() { 
	}

	VirtualFontImpl(int face, int style, int size) {

		this.size = size;

		switch (size) {
		case SIZE_LARGE:
			//inset = 4;
			font = FONT_8X16;
		case SIZE_MEDIUM:
			//inset = 0;
			font = FONT_8X16;
		case SIZE_SMALL:
			//inset = 0;
			font = FONT_8X16;
		}
	}

	public int charWidth(char ch) {
		return FONTS_WIDTH[font];
	}

	public int charsWidth(char[] ch, int offset, int length) {
		return length * FONTS_WIDTH[font];
	}

	public int getBaselinePosition() {
		return FONTS_HEIGHT[font] + inset / 2;
	}

	public int getFace() {
		return FACE_MONOSPACE;
	}

	public int getHeight() {
		return FONTS_HEIGHT[font] + inset;
	}

	public int getSize() {
		return size;
	}

	public int getStyle() {
		return STYLE_PLAIN;
	}

	public boolean isBold() {
		return false;
	}

	public boolean isItalic() {
		return false;
	}

	public boolean isPlain() {
		return true;
	}

	public boolean isUnderlined() {
		return false;
	}

	public int stringWidth(String str) {
		return str.length() * FONTS_WIDTH[font];
	}

	public int substringWidth(String str, int offset, int len) {
		return len * FONTS_WIDTH[font];
	}

	public void render(VirtualGraphics g, String str, int x, int y, int anchor) {
		x += g.getTranslateX();
		y += g.getTranslateY();

		if ((anchor & VirtualGraphics.BOTTOM) == VirtualGraphics.BOTTOM) {
			y -= getHeight() - 1;
		}

		if ((anchor & VirtualGraphics.RIGHT) == VirtualGraphics.RIGHT) {
			x -= stringWidth(str) - 1;
		} else if ((anchor & VirtualGraphics.HCENTER) == VirtualGraphics.HCENTER) {
			x -= stringWidth(str) / 2 - 1;
		}

		y += inset / 2;

		VirtualSurface surface = g.getSurface();

		if (Logging.TRACE_ENABLED)
			System.out.println("[DEBUG] RawFontPeer.render(): " + str + " x=" + x + " y=" + y + " color="
					+ Integer.toHexString(g.getColor()));

		int fw = FONTS_WIDTH[font];
		int fh = FONTS_HEIGHT[font];
		char data[] = FONTS_DATA[font];

		int color = g.getInternalColor();
		int pw = surface.getWidth();

		Rectangle r = g.clipRectangle;
		//System.out.println("clip rectangle : " + r + " fw=" + fw + " fh=" + fh);

		char[] chars = str.toCharArray();

		for (int i = 0; i < chars.length; i++) {

			int currentX = x + i * fw;

			// Set the destination offset to the next character
			int pk = y * surface.getWidth() + currentX;
			int dk = chars[i] * fh;
			
			// Check if character exists
			if (dk >= data.length) {
				continue;
			}

			// Trivial clipping
			if (currentX > r.xmax || y > r.ymax)
				break;

			// Pixel clipping
			int cxmin = 0, cxmax = fw;
			if (currentX < r.xmin)
				cxmin = r.xmin - currentX;
			if ((currentX + fw) > r.xmax) {
				cxmax = r.xmax - currentX;
				if (cxmax > fw)
					cxmax = fw;
			}
			int cymin = 0, cymax = fh;
			if (y < r.ymin)
				cymin = r.ymin - y;
			if ((y + fh) > r.ymax) {
				cymax = r.ymax - y;
				if (cymax > fh)
					cymax = fh;
			}

			//System.out.println(chars[i] + "  currentX=" + currentX + " cxmin=" + cxmin + " cxmax=" + cxmax + " cymin=" + cymin + " cymax=" + cymax);

			pk += pw * cymin;
			for (int cy = cymin; cy < cymax; cy++, pk += pw) {
				int d = data[dk + cy];
				int bit = 1 << (15 - cxmin);
				for (int cx = cxmin; cx < cxmax; cx++, bit >>= 1) {
					if ((d & bit) != 0) {
						surface.data[pk + cx] = color;
					}
				}
			}

		}

	}

	private final static char font8x8[] = { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7E00,
			0x8100, 0xA500, 0x8100, 0xBD00, 0x9900, 0x8100, 0x7E00, 0x7E00, 0xFF00, 0xDB00, 0xFF00, 0xC300, 0xE700,
			0xFF00, 0x7E00, 0x6C00, 0xFE00, 0xFE00, 0xFE00, 0x7C00, 0x3800, 0x1000, 0x0000, 0x1000, 0x3800, 0x7C00,
			0xFE00, 0x7C00, 0x3800, 0x1000, 0x0000, 0x3800, 0x7C00, 0x3800, 0xFE00, 0xFE00, 0xD600, 0x1000, 0x3800,
			0x1000, 0x3800, 0x7C00, 0xFE00, 0xFE00, 0x7C00, 0x1000, 0x3800, 0x0000, 0x0000, 0x1800, 0x3C00, 0x3C00,
			0x1800, 0x0000, 0x0000, 0xFF00, 0xFF00, 0xE700, 0xC300, 0xC300, 0xE700, 0xFF00, 0xFF00, 0x0000, 0x3C00,
			0x6600, 0x4200, 0x4200, 0x6600, 0x3C00, 0x0000, 0xFF00, 0xC300, 0x9900, 0xBD00, 0xBD00, 0x9900, 0xC300,
			0xFF00, 0x0F00, 0x0700, 0x0F00, 0x7D00, 0xCC00, 0xCC00, 0xCC00, 0x7800, 0x3C00, 0x6600, 0x6600, 0x6600,
			0x3C00, 0x1800, 0x7E00, 0x1800, 0x3F00, 0x3300, 0x3F00, 0x3000, 0x3000, 0x7000, 0xF000, 0xE000, 0x7F00,
			0x6300, 0x7F00, 0x6300, 0x6300, 0x6700, 0xE600, 0xC000, 0x1800, 0xDB00, 0x3C00, 0xE700, 0xE700, 0x3C00,
			0xDB00, 0x1800, 0x8000, 0xE000, 0xF800, 0xFE00, 0xF800, 0xE000, 0x8000, 0x0000, 0x0200, 0x0E00, 0x3E00,
			0xFE00, 0x3E00, 0x0E00, 0x0200, 0x0000, 0x1800, 0x3C00, 0x7E00, 0x1800, 0x1800, 0x7E00, 0x3C00, 0x1800,
			0x6600, 0x6600, 0x6600, 0x6600, 0x6600, 0x0000, 0x6600, 0x0000, 0x7F00, 0xDB00, 0xDB00, 0x7B00, 0x1B00,
			0x1B00, 0x1B00, 0x0000, 0x3E00, 0x6100, 0x3C00, 0x6600, 0x6600, 0x3C00, 0x8600, 0x7C00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x7E00, 0x7E00, 0x7E00, 0x0000, 0x1800, 0x3C00, 0x7E00, 0x1800, 0x7E00, 0x3C00, 0x1800,
			0xFF00, 0x1800, 0x3C00, 0x7E00, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x1800, 0x1800, 0x1800, 0x1800,
			0x7E00, 0x3C00, 0x1800, 0x0000, 0x0000, 0x1800, 0x0C00, 0xFE00, 0x0C00, 0x1800, 0x0000, 0x0000, 0x0000,
			0x3000, 0x6000, 0xFE00, 0x6000, 0x3000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC000, 0xC000, 0xC000, 0xFE00,
			0x0000, 0x0000, 0x0000, 0x2400, 0x6600, 0xFF00, 0x6600, 0x2400, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00,
			0x7E00, 0xFF00, 0xFF00, 0x0000, 0x0000, 0x0000, 0xFF00, 0xFF00, 0x7E00, 0x3C00, 0x1800, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00, 0x3C00, 0x1800, 0x1800,
			0x0000, 0x1800, 0x0000, 0x6600, 0x6600, 0x2400, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6C00, 0x6C00,
			0xFE00, 0x6C00, 0xFE00, 0x6C00, 0x6C00, 0x0000, 0x1800, 0x3E00, 0x6000, 0x3C00, 0x0600, 0x7C00, 0x1800,
			0x0000, 0x0000, 0xC600, 0xCC00, 0x1800, 0x3000, 0x6600, 0xC600, 0x0000, 0x3800, 0x6C00, 0x3800, 0x7600,
			0xDC00, 0xCC00, 0x7600, 0x0000, 0x1800, 0x1800, 0x3000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00,
			0x1800, 0x3000, 0x3000, 0x3000, 0x1800, 0x0C00, 0x0000, 0x3000, 0x1800, 0x0C00, 0x0C00, 0x0C00, 0x1800,
			0x3000, 0x0000, 0x0000, 0x6600, 0x3C00, 0xFF00, 0x3C00, 0x6600, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800,
			0x7E00, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x3000,
			0x0000, 0x0000, 0x0000, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1800, 0x1800, 0x0000, 0x0600, 0x0C00, 0x1800, 0x3000, 0x6000, 0xC000, 0x8000, 0x0000, 0x3800, 0x6C00,
			0xC600, 0xD600, 0xC600, 0x6C00, 0x3800, 0x0000, 0x1800, 0x3800, 0x1800, 0x1800, 0x1800, 0x1800, 0x7E00,
			0x0000, 0x7C00, 0xC600, 0x0600, 0x1C00, 0x3000, 0x6600, 0xFE00, 0x0000, 0x7C00, 0xC600, 0x0600, 0x3C00,
			0x0600, 0xC600, 0x7C00, 0x0000, 0x1C00, 0x3C00, 0x6C00, 0xCC00, 0xFE00, 0x0C00, 0x1E00, 0x0000, 0xFE00,
			0xC000, 0xC000, 0xFC00, 0x0600, 0xC600, 0x7C00, 0x0000, 0x3800, 0x6000, 0xC000, 0xFC00, 0xC600, 0xC600,
			0x7C00, 0x0000, 0xFE00, 0xC600, 0x0C00, 0x1800, 0x3000, 0x3000, 0x3000, 0x0000, 0x7C00, 0xC600, 0xC600,
			0x7C00, 0xC600, 0xC600, 0x7C00, 0x0000, 0x7C00, 0xC600, 0xC600, 0x7E00, 0x0600, 0x0C00, 0x7800, 0x0000,
			0x0000, 0x1800, 0x1800, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000,
			0x1800, 0x1800, 0x3000, 0x0600, 0x0C00, 0x1800, 0x3000, 0x1800, 0x0C00, 0x0600, 0x0000, 0x0000, 0x0000,
			0x7E00, 0x0000, 0x0000, 0x7E00, 0x0000, 0x0000, 0x6000, 0x3000, 0x1800, 0x0C00, 0x1800, 0x3000, 0x6000,
			0x0000, 0x7C00, 0xC600, 0x0C00, 0x1800, 0x1800, 0x0000, 0x1800, 0x0000, 0x7C00, 0xC600, 0xDE00, 0xDE00,
			0xDE00, 0xC000, 0x7800, 0x0000, 0x3800, 0x6C00, 0xC600, 0xFE00, 0xC600, 0xC600, 0xC600, 0x0000, 0xFC00,
			0x6600, 0x6600, 0x7C00, 0x6600, 0x6600, 0xFC00, 0x0000, 0x3C00, 0x6600, 0xC000, 0xC000, 0xC000, 0x6600,
			0x3C00, 0x0000, 0xF800, 0x6C00, 0x6600, 0x6600, 0x6600, 0x6C00, 0xF800, 0x0000, 0xFE00, 0x6200, 0x6800,
			0x7800, 0x6800, 0x6200, 0xFE00, 0x0000, 0xFE00, 0x6200, 0x6800, 0x7800, 0x6800, 0x6000, 0xF000, 0x0000,
			0x3C00, 0x6600, 0xC000, 0xC000, 0xCE00, 0x6600, 0x3A00, 0x0000, 0xC600, 0xC600, 0xC600, 0xFE00, 0xC600,
			0xC600, 0xC600, 0x0000, 0x3C00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x1E00, 0x0C00,
			0x0C00, 0x0C00, 0xCC00, 0xCC00, 0x7800, 0x0000, 0xE600, 0x6600, 0x6C00, 0x7800, 0x6C00, 0x6600, 0xE600,
			0x0000, 0xF000, 0x6000, 0x6000, 0x6000, 0x6200, 0x6600, 0xFE00, 0x0000, 0xC600, 0xEE00, 0xFE00, 0xFE00,
			0xD600, 0xC600, 0xC600, 0x0000, 0xC600, 0xE600, 0xF600, 0xDE00, 0xCE00, 0xC600, 0xC600, 0x0000, 0x7C00,
			0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0xFC00, 0x6600, 0x6600, 0x7C00, 0x6000, 0x6000,
			0xF000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600, 0xCE00, 0x7C00, 0x0E00, 0xFC00, 0x6600, 0x6600,
			0x7C00, 0x6C00, 0x6600, 0xE600, 0x0000, 0x3C00, 0x6600, 0x3000, 0x1800, 0x0C00, 0x6600, 0x3C00, 0x0000,
			0x7E00, 0x7E00, 0x5A00, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600,
			0xC600, 0x7C00, 0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x6C00, 0x3800, 0x0000, 0xC600, 0xC600,
			0xC600, 0xD600, 0xD600, 0xFE00, 0x6C00, 0x0000, 0xC600, 0xC600, 0x6C00, 0x3800, 0x6C00, 0xC600, 0xC600,
			0x0000, 0x6600, 0x6600, 0x6600, 0x3C00, 0x1800, 0x1800, 0x3C00, 0x0000, 0xFE00, 0xC600, 0x8C00, 0x1800,
			0x3200, 0x6600, 0xFE00, 0x0000, 0x3C00, 0x3000, 0x3000, 0x3000, 0x3000, 0x3000, 0x3C00, 0x0000, 0xC000,
			0x6000, 0x3000, 0x1800, 0x0C00, 0x0600, 0x0200, 0x0000, 0x3C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00,
			0x3C00, 0x0000, 0x1000, 0x3800, 0x6C00, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0x3000, 0x1800, 0x0C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0x7600, 0x0000, 0xE000, 0x6000, 0x7C00, 0x6600, 0x6600,
			0x6600, 0xDC00, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC000, 0xC600, 0x7C00, 0x0000, 0x1C00, 0x0C00,
			0x7C00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xFE00, 0xC000, 0x7C00,
			0x0000, 0x3C00, 0x6600, 0x6000, 0xF800, 0x6000, 0x6000, 0xF000, 0x0000, 0x0000, 0x0000, 0x7600, 0xCC00,
			0xCC00, 0x7C00, 0x0C00, 0xF800, 0xE000, 0x6000, 0x6C00, 0x7600, 0x6600, 0x6600, 0xE600, 0x0000, 0x1800,
			0x0000, 0x3800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0600, 0x0000, 0x0600, 0x0600, 0x0600, 0x6600,
			0x6600, 0x3C00, 0xE000, 0x6000, 0x6600, 0x6C00, 0x7800, 0x6C00, 0xE600, 0x0000, 0x3800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0xEC00, 0xFE00, 0xD600, 0xD600, 0xD600, 0x0000,
			0x0000, 0x0000, 0xDC00, 0x6600, 0x6600, 0x6600, 0x6600, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600,
			0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0xDC00, 0x6600, 0x6600, 0x7C00, 0x6000, 0xF000, 0x0000, 0x0000,
			0x7600, 0xCC00, 0xCC00, 0x7C00, 0x0C00, 0x1E00, 0x0000, 0x0000, 0xDC00, 0x7600, 0x6000, 0x6000, 0xF000,
			0x0000, 0x0000, 0x0000, 0x7E00, 0xC000, 0x7C00, 0x0600, 0xFC00, 0x0000, 0x3000, 0x3000, 0xFC00, 0x3000,
			0x3000, 0x3600, 0x1C00, 0x0000, 0x0000, 0x0000, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000,
			0x0000, 0xC600, 0xC600, 0xC600, 0x6C00, 0x3800, 0x0000, 0x0000, 0x0000, 0xC600, 0xD600, 0xD600, 0xFE00,
			0x6C00, 0x0000, 0x0000, 0x0000, 0xC600, 0x6C00, 0x3800, 0x6C00, 0xC600, 0x0000, 0x0000, 0x0000, 0xC600,
			0xC600, 0xC600, 0x7E00, 0x0600, 0xFC00, 0x0000, 0x0000, 0x7E00, 0x4C00, 0x1800, 0x3200, 0x7E00, 0x0000,
			0x0E00, 0x1800, 0x1800, 0x7000, 0x1800, 0x1800, 0x0E00, 0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x0000, 0x7000, 0x1800, 0x1800, 0x0E00, 0x1800, 0x1800, 0x7000, 0x0000, 0x7600, 0xDC00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x6C00, 0xC600, 0xC600, 0xFE00,
			0x0000, 0x7C00, 0xC600, 0xC000, 0xC000, 0xC600, 0x7C00, 0x0C00, 0x7800, 0xCC00, 0x0000, 0xCC00, 0xCC00,
			0xCC00, 0xCC00, 0x7600, 0x0000, 0x0C00, 0x1800, 0x7C00, 0xC600, 0xFE00, 0xC000, 0x7C00, 0x0000, 0x7C00,
			0x8200, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0x7600, 0x0000, 0xC600, 0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00,
			0x7600, 0x0000, 0x3000, 0x1800, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0x7600, 0x0000, 0x3000, 0x3000, 0x7800,
			0x0C00, 0x7C00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x7E00, 0xC000, 0xC000, 0x7E00, 0x0C00, 0x3800,
			0x7C00, 0x8200, 0x7C00, 0xC600, 0xFE00, 0xC000, 0x7C00, 0x0000, 0xC600, 0x0000, 0x7C00, 0xC600, 0xFE00,
			0xC000, 0x7C00, 0x0000, 0x3000, 0x1800, 0x7C00, 0xC600, 0xFE00, 0xC000, 0x7C00, 0x0000, 0x6600, 0x0000,
			0x3800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x7C00, 0x8200, 0x3800, 0x1800, 0x1800, 0x1800, 0x3C00,
			0x0000, 0x3000, 0x1800, 0x0000, 0x3800, 0x1800, 0x1800, 0x3C00, 0x0000, 0xC600, 0x3800, 0x6C00, 0xC600,
			0xFE00, 0xC600, 0xC600, 0x0000, 0x3800, 0x6C00, 0x7C00, 0xC600, 0xFE00, 0xC600, 0xC600, 0x0000, 0x1800,
			0x3000, 0xFE00, 0xC000, 0xF800, 0xC000, 0xFE00, 0x0000, 0x0000, 0x0000, 0x7E00, 0x1800, 0x7E00, 0xD800,
			0x7E00, 0x0000, 0x3E00, 0x6C00, 0xCC00, 0xFE00, 0xCC00, 0xCC00, 0xCE00, 0x0000, 0x7C00, 0x8200, 0x7C00,
			0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0xC600, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000,
			0x3000, 0x1800, 0x7C00, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x7800, 0x8400, 0x0000, 0xCC00, 0xCC00,
			0xCC00, 0x7600, 0x0000, 0x6000, 0x3000, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0xC600, 0x0000,
			0xC600, 0xC600, 0xC600, 0x7E00, 0x0600, 0xFC00, 0xC600, 0x3800, 0x6C00, 0xC600, 0xC600, 0x6C00, 0x3800,
			0x0000, 0xC600, 0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x1800, 0x1800, 0x7E00, 0xC000,
			0xC000, 0x7E00, 0x1800, 0x1800, 0x3800, 0x6C00, 0x6400, 0xF000, 0x6000, 0x6600, 0xFC00, 0x0000, 0x6600,
			0x6600, 0x3C00, 0x7E00, 0x1800, 0x7E00, 0x1800, 0x1800, 0xF800, 0xCC00, 0xCC00, 0xFA00, 0xC600, 0xCF00,
			0xC600, 0xC700, 0x0E00, 0x1B00, 0x1800, 0x3C00, 0x1800, 0xD800, 0x7000, 0x0000, 0x1800, 0x3000, 0x7800,
			0x0C00, 0x7C00, 0xCC00, 0x7600, 0x0000, 0x0C00, 0x1800, 0x0000, 0x3800, 0x1800, 0x1800, 0x3C00, 0x0000,
			0x0C00, 0x1800, 0x7C00, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x1800, 0x3000, 0xCC00, 0xCC00, 0xCC00,
			0xCC00, 0x7600, 0x0000, 0x7600, 0xDC00, 0x0000, 0xDC00, 0x6600, 0x6600, 0x6600, 0x0000, 0x7600, 0xDC00,
			0x0000, 0xE600, 0xF600, 0xDE00, 0xCE00, 0x0000, 0x3C00, 0x6C00, 0x6C00, 0x3E00, 0x0000, 0x7E00, 0x0000,
			0x0000, 0x3800, 0x6C00, 0x6C00, 0x3800, 0x0000, 0x7C00, 0x0000, 0x0000, 0x1800, 0x0000, 0x1800, 0x1800,
			0x3000, 0x6300, 0x3E00, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0xC000, 0xC000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xFE00, 0x0600, 0x0600, 0x0000, 0x0000, 0x6300, 0xE600, 0x6C00, 0x7E00, 0x3300, 0x6600,
			0xCC00, 0x0F00, 0x6300, 0xE600, 0x6C00, 0x7A00, 0x3600, 0x6A00, 0xDF00, 0x0600, 0x1800, 0x0000, 0x1800,
			0x1800, 0x3C00, 0x3C00, 0x1800, 0x0000, 0x0000, 0x3300, 0x6600, 0xCC00, 0x6600, 0x3300, 0x0000, 0x0000,
			0x0000, 0xCC00, 0x6600, 0x3300, 0x6600, 0xCC00, 0x0000, 0x0000, 0x2200, 0x8800, 0x2200, 0x8800, 0x2200,
			0x8800, 0x2200, 0x8800, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x7700, 0xDD00,
			0x7700, 0xDD00, 0x7700, 0xDD00, 0x7700, 0xDD00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xF800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xF800, 0x1800,
			0xF800, 0x1800, 0x1800, 0x1800, 0x3600, 0x3600, 0x3600, 0x3600, 0xF600, 0x3600, 0x3600, 0x3600, 0x0000,
			0x0000, 0x0000, 0x0000, 0xFE00, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0xF800, 0x1800, 0xF800, 0x1800,
			0x1800, 0x1800, 0x3600, 0x3600, 0xF600, 0x0600, 0xF600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0xFE00, 0x0600, 0xF600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0xF600, 0x0600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x3600, 0x3600, 0x3600, 0x3600, 0xFE00,
			0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0xF800, 0x1800, 0xF800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xF800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1F00, 0x0000, 0x0000,
			0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xFF00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1F00, 0x1800, 0x1800, 0x1800, 0x0000,
			0x0000, 0x0000, 0x0000, 0xFF00, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1F00, 0x1800, 0x1F00, 0x1800, 0x1800, 0x1800, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3700, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3700, 0x3000, 0x3F00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x3F00, 0x3000, 0x3700, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xF700, 0x0000, 0xFF00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0x0000, 0xF700, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3700, 0x3000, 0x3700, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0xFF00, 0x0000, 0xFF00, 0x0000, 0x0000,
			0x0000, 0x3600, 0x3600, 0xF700, 0x0000, 0xF700, 0x3600, 0x3600, 0x3600, 0x1800, 0x1800, 0xFF00, 0x0000,
			0xFF00, 0x0000, 0x0000, 0x0000, 0x3600, 0x3600, 0x3600, 0x3600, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xFF00, 0x0000, 0xFF00, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3F00, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x1F00,
			0x1800, 0x1F00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1F00, 0x1800, 0x1F00, 0x1800, 0x1800, 0x1800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x3F00, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xFF00,
			0x3600, 0x3600, 0x3600, 0x1800, 0x1800, 0xFF00, 0x1800, 0xFF00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0xF800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1F00, 0x1800, 0x1800,
			0x1800, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000,
			0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0x0F00,
			0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0xC800, 0xDC00, 0x7600, 0x0000, 0x7800, 0xCC00, 0xCC00,
			0xD800, 0xCC00, 0xC600, 0xCC00, 0x0000, 0xFE00, 0xC600, 0xC000, 0xC000, 0xC000, 0xC000, 0xC000, 0x0000,
			0x0000, 0x0000, 0xFE00, 0x6C00, 0x6C00, 0x6C00, 0x6C00, 0x0000, 0xFE00, 0xC600, 0x6000, 0x3000, 0x6000,
			0xC600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x7E00, 0xD800, 0xD800, 0xD800, 0x7000, 0x0000, 0x0000, 0x0000,
			0x6600, 0x6600, 0x6600, 0x6600, 0x7C00, 0xC000, 0x0000, 0x7600, 0xDC00, 0x1800, 0x1800, 0x1800, 0x1800,
			0x0000, 0x7E00, 0x1800, 0x3C00, 0x6600, 0x6600, 0x3C00, 0x1800, 0x7E00, 0x3800, 0x6C00, 0xC600, 0xFE00,
			0xC600, 0x6C00, 0x3800, 0x0000, 0x3800, 0x6C00, 0xC600, 0xC600, 0x6C00, 0x6C00, 0xEE00, 0x0000, 0x0E00,
			0x1800, 0x0C00, 0x3E00, 0x6600, 0x6600, 0x3C00, 0x0000, 0x0000, 0x0000, 0x7E00, 0xDB00, 0xDB00, 0x7E00,
			0x0000, 0x0000, 0x0600, 0x0C00, 0x7E00, 0xDB00, 0xDB00, 0x7E00, 0x6000, 0xC000, 0x1E00, 0x3000, 0x6000,
			0x7E00, 0x6000, 0x3000, 0x1E00, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x0000,
			0x0000, 0xFE00, 0x0000, 0xFE00, 0x0000, 0xFE00, 0x0000, 0x0000, 0x1800, 0x1800, 0x7E00, 0x1800, 0x1800,
			0x0000, 0x7E00, 0x0000, 0x3000, 0x1800, 0x0C00, 0x1800, 0x3000, 0x0000, 0x7E00, 0x0000, 0x0C00, 0x1800,
			0x3000, 0x1800, 0x0C00, 0x0000, 0x7E00, 0x0000, 0x0E00, 0x1B00, 0x1B00, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xD800, 0xD800, 0x7000, 0x0000, 0x1800, 0x0000, 0x7E00,
			0x0000, 0x1800, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0x0000, 0x7600, 0xDC00, 0x0000, 0x0000, 0x3800,
			0x6C00, 0x6C00, 0x3800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0F00, 0x0C00, 0x0C00,
			0x0C00, 0xEC00, 0x6C00, 0x3C00, 0x1C00, 0x6C00, 0x3600, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0x0000,
			0x7800, 0x0C00, 0x1800, 0x3000, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x3C00, 0x3C00,
			0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, };

	private final static char font8x16[] = { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7E00, 0x8100, 0xA500, 0x8100,
			0x8100, 0xBD00, 0x9900, 0x8100, 0x8100, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7E00,
			0xFF00, 0xDB00, 0xFF00, 0xFF00, 0xC300, 0xE700, 0xFF00, 0xFF00, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x6C00, 0xFE00, 0xFE00, 0xFE00, 0xFE00, 0x7C00, 0x3800, 0x1000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x7C00, 0xFE00, 0x7C00, 0x3800,
			0x1000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00, 0x3C00, 0xE700,
			0xE700, 0xE700, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800,
			0x3C00, 0x7E00, 0xFF00, 0xFF00, 0x7E00, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00, 0x3C00, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xE700, 0xC300, 0xC300, 0xE700, 0xFF00,
			0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x6600, 0x4200,
			0x4200, 0x6600, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00,
			0xC300, 0x9900, 0xBD00, 0xBD00, 0x9900, 0xC300, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0x0000, 0x0000,
			0x1E00, 0x0E00, 0x1A00, 0x3200, 0x7800, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x3C00, 0x6600, 0x6600, 0x6600, 0x6600, 0x3C00, 0x1800, 0x7E00, 0x1800, 0x1800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3F00, 0x3300, 0x3F00, 0x3000, 0x3000, 0x3000, 0x3000,
			0x7000, 0xF000, 0xE000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7F00, 0x6300, 0x7F00, 0x6300,
			0x6300, 0x6300, 0x6300, 0x6700, 0xE700, 0xE600, 0xC000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1800, 0x1800, 0xDB00, 0x3C00, 0xE700, 0x3C00, 0xDB00, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x8000, 0xC000, 0xE000, 0xF000, 0xF800, 0xFE00, 0xF800, 0xF000, 0xE000, 0xC000, 0x8000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0200, 0x0600, 0x0E00, 0x1E00, 0x3E00, 0xFE00, 0x3E00, 0x1E00, 0x0E00,
			0x0600, 0x0200, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00, 0x7E00, 0x1800, 0x1800,
			0x1800, 0x7E00, 0x3C00, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6600, 0x6600,
			0x6600, 0x6600, 0x6600, 0x6600, 0x6600, 0x0000, 0x6600, 0x6600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x7F00, 0xDB00, 0xDB00, 0xDB00, 0x7B00, 0x1B00, 0x1B00, 0x1B00, 0x1B00, 0x1B00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0x6000, 0x3800, 0x6C00, 0xC600, 0xC600, 0x6C00, 0x3800, 0x0C00,
			0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xFE00, 0xFE00, 0xFE00, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00, 0x7E00,
			0x1800, 0x1800, 0x1800, 0x7E00, 0x3C00, 0x1800, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1800, 0x3C00, 0x7E00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x7E00, 0x3C00, 0x1800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x0C00, 0xFE00, 0x0C00,
			0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3000,
			0x6000, 0xFE00, 0x6000, 0x3000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0xC000, 0xC000, 0xC000, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x2800, 0x6C00, 0xFE00, 0x6C00, 0x2800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x3800, 0x7C00, 0x7C00, 0xFE00,
			0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0xFE00, 0x7C00,
			0x7C00, 0x3800, 0x3800, 0x1000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x1800, 0x3C00, 0x3C00, 0x3C00, 0x1800, 0x1800, 0x1800, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x6600, 0x6600, 0x6600, 0x2400, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6C00, 0x6C00, 0xFE00, 0x6C00, 0x6C00,
			0x6C00, 0xFE00, 0x6C00, 0x6C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x7C00, 0xC600, 0xC200,
			0xC000, 0x7C00, 0x0600, 0x0600, 0x8600, 0xC600, 0x7C00, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xC200, 0xC600, 0x0C00, 0x1800, 0x3000, 0x6000, 0xC600, 0x8600, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0x6C00, 0x3800, 0x7600, 0xDC00, 0xCC00, 0xCC00, 0xCC00, 0x7600,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3000, 0x3000, 0x3000, 0x6000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00, 0x1800, 0x3000, 0x3000,
			0x3000, 0x3000, 0x3000, 0x3000, 0x1800, 0x0C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3000,
			0x1800, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x1800, 0x3000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6600, 0x3C00, 0xFF00, 0x3C00, 0x6600, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x7E00, 0x1800, 0x1800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x1800, 0x1800, 0x1800, 0x3000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0200, 0x0600, 0x0C00, 0x1800, 0x3000, 0x6000, 0xC000,
			0x8000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0xC600, 0xC600, 0xD600, 0xD600,
			0xC600, 0xC600, 0x6C00, 0x3800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3800, 0x7800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x7C00, 0xC600, 0x0600, 0x0C00, 0x1800, 0x3000, 0x6000, 0xC000, 0xC600, 0xFE00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0x0600, 0x0600, 0x3C00, 0x0600, 0x0600, 0x0600, 0xC600, 0x7C00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00, 0x1C00, 0x3C00, 0x6C00, 0xCC00, 0xFE00, 0x0C00,
			0x0C00, 0x0C00, 0x1E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0xC000, 0xC000, 0xC000,
			0xFC00, 0x0600, 0x0600, 0x0600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3800,
			0x6000, 0xC000, 0xC000, 0xFC00, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xFE00, 0xC600, 0x0600, 0x0600, 0x0C00, 0x1800, 0x3000, 0x3000, 0x3000, 0x3000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0x7C00, 0xC600, 0xC600, 0xC600,
			0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0x7E00,
			0x0600, 0x0600, 0x0600, 0x0C00, 0x7800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x3000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0600, 0x0C00, 0x1800, 0x3000, 0x6000, 0x3000, 0x1800, 0x0C00,
			0x0600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7E00, 0x0000, 0x0000,
			0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6000, 0x3000,
			0x1800, 0x0C00, 0x0600, 0x0C00, 0x1800, 0x3000, 0x6000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x7C00, 0xC600, 0xC600, 0x0C00, 0x1800, 0x1800, 0x1800, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xDE00, 0xDE00, 0xDE00, 0xDC00, 0xC000, 0x7C00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x6C00, 0xC600, 0xC600, 0xFE00, 0xC600,
			0xC600, 0xC600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFC00, 0x6600, 0x6600, 0x6600,
			0x7C00, 0x6600, 0x6600, 0x6600, 0x6600, 0xFC00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00,
			0x6600, 0xC200, 0xC000, 0xC000, 0xC000, 0xC000, 0xC200, 0x6600, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xF800, 0x6C00, 0x6600, 0x6600, 0x6600, 0x6600, 0x6600, 0x6600, 0x6C00, 0xF800, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0x6600, 0x6200, 0x6800, 0x7800, 0x6800, 0x6000, 0x6200,
			0x6600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0x6600, 0x6200, 0x6800, 0x7800,
			0x6800, 0x6000, 0x6000, 0x6000, 0xF000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x6600,
			0xC200, 0xC000, 0xC000, 0xDE00, 0xC600, 0xC600, 0x6600, 0x3A00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xFE00, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1E00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00,
			0xCC00, 0xCC00, 0xCC00, 0x7800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xE600, 0x6600, 0x6600,
			0x6C00, 0x7800, 0x7800, 0x6C00, 0x6600, 0x6600, 0xE600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xF000, 0x6000, 0x6000, 0x6000, 0x6000, 0x6000, 0x6000, 0x6200, 0x6600, 0xFE00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0xC600, 0xEE00, 0xFE00, 0xFE00, 0xD600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0xE600, 0xF600, 0xFE00, 0xDE00, 0xCE00, 0xC600,
			0xC600, 0xC600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600,
			0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFC00,
			0x6600, 0x6600, 0x6600, 0x7C00, 0x6000, 0x6000, 0x6000, 0x6000, 0xF000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xD600, 0xDE00, 0x7C00, 0x0C00,
			0x0E00, 0x0000, 0x0000, 0x0000, 0x0000, 0xFC00, 0x6600, 0x6600, 0x6600, 0x7C00, 0x6C00, 0x6600, 0x6600,
			0x6600, 0xE600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0x6000, 0x3800,
			0x0C00, 0x0600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7E00, 0x7E00,
			0x5A00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x6C00, 0x3800,
			0x1000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xD600, 0xD600,
			0xD600, 0xFE00, 0xEE00, 0x6C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0xC600, 0x6C00,
			0x7C00, 0x3800, 0x3800, 0x7C00, 0x6C00, 0xC600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x6600, 0x6600, 0x6600, 0x6600, 0x3C00, 0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0xFE00, 0xC600, 0x8600, 0x0C00, 0x1800, 0x3000, 0x6000, 0xC200, 0xC600, 0xFE00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x3000, 0x3000, 0x3000, 0x3000, 0x3000, 0x3000,
			0x3000, 0x3000, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x8000, 0xC000, 0xE000,
			0x7000, 0x3800, 0x1C00, 0x0E00, 0x0600, 0x0200, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00,
			0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1000, 0x3800, 0x6C00, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0xFF00, 0x0000, 0x0000, 0x0000, 0x3000, 0x1800, 0x0C00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xE000, 0x6000, 0x6000, 0x7800, 0x6C00, 0x6600, 0x6600, 0x6600, 0x6600, 0x7C00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC000, 0xC000, 0xC000, 0xC600,
			0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1C00, 0x0C00, 0x0C00, 0x3C00, 0x6C00, 0xCC00,
			0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x7C00, 0xC600, 0xFE00, 0xC000, 0xC000, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1C00, 0x3600, 0x3200, 0x3000, 0x7800, 0x3000, 0x3000, 0x3000, 0x3000, 0x7800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7C00,
			0x0C00, 0xCC00, 0x7800, 0x0000, 0x0000, 0x0000, 0xE000, 0x6000, 0x6000, 0x6C00, 0x7600, 0x6600, 0x6600,
			0x6600, 0x6600, 0xE600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x3800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0600,
			0x0600, 0x0000, 0x0E00, 0x0600, 0x0600, 0x0600, 0x0600, 0x0600, 0x0600, 0x6600, 0x6600, 0x3C00, 0x0000,
			0x0000, 0x0000, 0xE000, 0x6000, 0x6000, 0x6600, 0x6C00, 0x7800, 0x7800, 0x6C00, 0x6600, 0xE600, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xEC00, 0xFE00,
			0xD600, 0xD600, 0xD600, 0xD600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xDC00, 0x6600, 0x6600, 0x6600, 0x6600, 0x6600, 0x6600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xDC00, 0x6600, 0x6600, 0x6600, 0x6600, 0x6600,
			0x7C00, 0x6000, 0x6000, 0xF000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xCC00, 0xCC00,
			0xCC00, 0xCC00, 0xCC00, 0x7C00, 0x0C00, 0x0C00, 0x1E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xDC00, 0x7600, 0x6600, 0x6000, 0x6000, 0x6000, 0xF000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0x6000, 0x3800, 0x0C00, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x1000, 0x3000, 0x3000, 0xFC00, 0x3000, 0x3000, 0x3000, 0x3000, 0x3600, 0x1C00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xCC00, 0xCC00, 0xCC00, 0xCC00,
			0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600,
			0xC600, 0xC600, 0xC600, 0xC600, 0x6C00, 0x3800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xC600, 0xC600, 0xD600, 0xD600, 0xD600, 0xFE00, 0x6C00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0x6C00, 0x3800, 0x3800, 0x3800, 0x6C00, 0xC600, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600,
			0xC600, 0x7E00, 0x0600, 0x0C00, 0xF800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0xCC00,
			0x1800, 0x3000, 0x6000, 0xC600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0E00, 0x1800,
			0x1800, 0x1800, 0x7000, 0x1800, 0x1800, 0x1800, 0x1800, 0x0E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x7000, 0x1800, 0x1800, 0x1800, 0x0E00, 0x1800, 0x1800, 0x1800, 0x1800,
			0x7000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1000,
			0x3800, 0x6C00, 0xC600, 0xC600, 0xC600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x3C00, 0x6600, 0xC200, 0xC000, 0xC000, 0xC000, 0xC000, 0xC200, 0x6600, 0x3C00, 0x1800, 0x7000, 0x0000,
			0x0000, 0x0000, 0x0000, 0xCC00, 0x0000, 0x0000, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7600,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00, 0x1800, 0x3000, 0x0000, 0x7C00, 0xC600, 0xFE00, 0xC000,
			0xC000, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x6C00, 0x0000, 0x7800,
			0x0C00, 0x7C00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xCC00,
			0x0000, 0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x6000, 0x3000, 0x1800, 0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0x3800, 0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0xCC00,
			0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600,
			0xC000, 0xC000, 0xC000, 0xC600, 0x7C00, 0x1800, 0x7000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x6C00,
			0x0000, 0x7C00, 0xC600, 0xFE00, 0xC000, 0xC000, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xC600, 0x0000, 0x0000, 0x7C00, 0xC600, 0xFE00, 0xC000, 0xC000, 0xC600, 0x7C00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x6000, 0x3000, 0x1800, 0x0000, 0x7C00, 0xC600, 0xFE00, 0xC000, 0xC000, 0xC600,
			0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6600, 0x0000, 0x0000, 0x3800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3C00, 0x6600, 0x0000,
			0x3800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6000,
			0x3000, 0x1800, 0x0000, 0x3800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xC600, 0x0000, 0x1000, 0x3800, 0x6C00, 0xC600, 0xC600, 0xFE00, 0xC600, 0xC600, 0xC600,
			0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0x3800, 0x1000, 0x3800, 0x6C00, 0xC600, 0xFE00, 0xC600,
			0xC600, 0xC600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00, 0x1800, 0x0000, 0xFE00, 0x6600, 0x6200,
			0x6800, 0x7800, 0x6800, 0x6200, 0x6600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xEC00, 0x3600, 0x3600, 0x7E00, 0xD800, 0xD800, 0x6E00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x3E00, 0x6C00, 0xCC00, 0xCC00, 0xFE00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCE00, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x1000, 0x3800, 0x6C00, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600,
			0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0x0000, 0x0000, 0x7C00, 0xC600,
			0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6000, 0x3000, 0x1800,
			0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x3000, 0x7800, 0xCC00, 0x0000, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x6000, 0x3000, 0x1800, 0x0000, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00,
			0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600, 0x0000, 0x0000, 0xC600, 0xC600, 0xC600,
			0xC600, 0xC600, 0xC600, 0x7E00, 0x0600, 0x0C00, 0x7800, 0x0000, 0x0000, 0xC600, 0x0000, 0x7C00, 0xC600,
			0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xC600,
			0x0000, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x1800, 0x1800, 0x7C00, 0xC600, 0xC000, 0xC000, 0xC000, 0xC600, 0x7C00, 0x1800, 0x1800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0x6400, 0x6000, 0xF000, 0x6000, 0x6000, 0x6000,
			0x6000, 0xE600, 0xFC00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6600, 0x6600, 0x3C00, 0x1800,
			0x7E00, 0x1800, 0x7E00, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xF800, 0xCC00,
			0xCC00, 0xF800, 0xC400, 0xCC00, 0xDE00, 0xCC00, 0xCC00, 0xCC00, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0E00, 0x1B00, 0x1800, 0x1800, 0x1800, 0x7E00, 0x1800, 0x1800, 0x1800, 0xD800, 0x7000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3000, 0x6000, 0x0000, 0x7800, 0x0C00, 0x7C00, 0xCC00, 0xCC00,
			0xCC00, 0x7600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00, 0x1800, 0x3000, 0x0000, 0x3800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x3000, 0x6000,
			0x0000, 0x7C00, 0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x1800, 0x3000, 0x6000, 0x0000, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0xCC00, 0x7600, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0x0000, 0xDC00, 0x6600, 0x6600, 0x6600, 0x6600, 0x6600,
			0x6600, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0x0000, 0xC600, 0xE600, 0xF600, 0xFE00, 0xDE00,
			0xCE00, 0xC600, 0xC600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x6C00, 0x6C00,
			0x3E00, 0x0000, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x3800, 0x6C00, 0x6C00, 0x3800, 0x0000, 0x7C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x3000, 0x3000, 0x0000, 0x3000, 0x3000, 0x6000, 0xC000, 0xC600, 0xC600, 0x7C00,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0xC000, 0xC000,
			0xC000, 0xC000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xFE00, 0x0600, 0x0600, 0x0600, 0x0600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6000, 0xE000,
			0x6200, 0x6600, 0x6C00, 0x1800, 0x3000, 0x6000, 0xDC00, 0x8600, 0x0C00, 0x1800, 0x3E00, 0x0000, 0x0000,
			0x0000, 0x6000, 0xE000, 0x6200, 0x6600, 0x6C00, 0x1800, 0x3000, 0x6600, 0xCE00, 0x9A00, 0x3F00, 0x0600,
			0x0600, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x1800, 0x1800, 0x1800, 0x3C00, 0x3C00,
			0x3C00, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3600, 0x6C00,
			0xD800, 0x6C00, 0x3600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xD800, 0x6C00, 0x3600, 0x6C00, 0xD800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1100,
			0x4400, 0x1100, 0x4400, 0x1100, 0x4400, 0x1100, 0x4400, 0x1100, 0x4400, 0x1100, 0x4400, 0x1100, 0x4400,
			0x1100, 0x4400, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x5500, 0xAA00, 0x5500,
			0xAA00, 0x5500, 0xAA00, 0x5500, 0xAA00, 0xDD00, 0x7700, 0xDD00, 0x7700, 0xDD00, 0x7700, 0xDD00, 0x7700,
			0xDD00, 0x7700, 0xDD00, 0x7700, 0xDD00, 0x7700, 0xDD00, 0x7700, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xF800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xF800, 0x1800, 0xF800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xF600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xFE00, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0xF800, 0x1800, 0xF800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xF600, 0x0600, 0xF600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0x0600,
			0xF600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0xF600, 0x0600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xF800, 0x1800, 0xF800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xF800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1F00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1F00, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1F00, 0x1800, 0x1F00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3700, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3700, 0x3000,
			0x3F00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x3F00, 0x3000, 0x3700, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0xF700, 0x0000, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0x0000, 0xF700, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3700, 0x3000, 0x3700,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xFF00, 0x0000, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0xF700, 0x0000, 0xF700, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0x0000, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xFF00, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00,
			0x0000, 0xFF00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3F00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1F00, 0x1800, 0x1F00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1F00, 0x1800,
			0x1F00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x3F00, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0xFF00, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x3600, 0x3600, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0x1800, 0xFF00, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xF800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x1F00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xFF00, 0xFF00,
			0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00,
			0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00,
			0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000,
			0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0xF000, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00,
			0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0x0F00, 0xFF00, 0xFF00, 0xFF00,
			0xFF00, 0xFF00, 0xFF00, 0xFF00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0xD800, 0xD800, 0xD800, 0xDC00, 0x7600, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7800, 0xCC00, 0xCC00, 0xCC00, 0xD800, 0xCC00, 0xC600, 0xC600,
			0xC600, 0xCC00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFE00, 0xC600, 0xC600, 0xC000, 0xC000,
			0xC000, 0xC000, 0xC000, 0xC000, 0xC000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xFE00, 0x6C00, 0x6C00, 0x6C00, 0x6C00, 0x6C00, 0x6C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0xFE00, 0xC600, 0x6000, 0x3000, 0x1800, 0x1800, 0x3000, 0x6000, 0xC600, 0xFE00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7E00, 0xD800, 0xD800, 0xD800, 0xD800, 0xD800,
			0x7000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6600, 0x6600, 0x6600,
			0x6600, 0x6600, 0x6600, 0x7C00, 0x6000, 0x6000, 0xC000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600,
			0xDC00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x7E00, 0x1800, 0x3C00, 0x6600, 0x6600, 0x6600, 0x6600, 0x3C00, 0x1800, 0x7E00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0xC600, 0xC600, 0xFE00, 0xC600, 0xC600, 0xC600, 0x6C00, 0x3800,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0xC600, 0xC600, 0xC600, 0x6C00, 0x6C00,
			0x6C00, 0x6C00, 0xEE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1E00, 0x3000, 0x1800, 0x0C00,
			0x3E00, 0x6600, 0x6600, 0x6600, 0x6600, 0x3C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x7E00, 0xDB00, 0xDB00, 0xDB00, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0300, 0x0600, 0x7E00, 0xDB00, 0xDB00, 0xF300, 0x7E00, 0x6000, 0xC000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1C00, 0x3000, 0x6000, 0x6000, 0x7C00, 0x6000, 0x6000, 0x6000,
			0x3000, 0x1C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7C00, 0xC600, 0xC600, 0xC600,
			0xC600, 0xC600, 0xC600, 0xC600, 0xC600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0xFE00, 0x0000, 0x0000, 0xFE00, 0x0000, 0x0000, 0xFE00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x7E00, 0x1800, 0x1800, 0x0000, 0x0000, 0x7E00, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3000, 0x1800, 0x0C00, 0x0600, 0x0C00, 0x1800, 0x3000, 0x0000,
			0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0C00, 0x1800, 0x3000, 0x6000, 0x3000,
			0x1800, 0x0C00, 0x0000, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0E00, 0x1B00, 0x1B00,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800,
			0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0xD800, 0xD800, 0xD800, 0x7000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x0000, 0x7E00, 0x0000, 0x1800, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7600, 0xDC00, 0x0000, 0x7600,
			0xDC00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3800, 0x6C00, 0x6C00, 0x3800, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x1800, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0F00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0x0C00, 0xEC00, 0x6C00, 0x6C00,
			0x3C00, 0x1C00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x6C00, 0x3600, 0x3600, 0x3600, 0x3600, 0x3600,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x3C00, 0x6600, 0x0C00,
			0x1800, 0x3200, 0x7E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x7E00, 0x7E00, 0x7E00, 0x7E00, 0x7E00, 0x7E00, 0x7E00, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
			0x0000, 0x0000, 0x0000, 0x0000, 0x0000, };

	private static final int FONTS_WIDTH[] = { 8, 8, 16 };
	private static final int FONTS_HEIGHT[] = { 8, 16, 16 };
	private static final char FONTS_DATA[][] = { font8x8, font8x16 };

}
